#
# Copyright (C) 2019 Satoru SATOH <satoru.satoh@gmail.com>
# License: MIT
#
# pylint: disable=unused-import
r"""Load and parse MHTML data.

.. versionadded:: 0.0.1
"""

import email, logging, re

from base64 import b64encode
from bs4 import BeautifulSoup
from pathlib import Path
from urllib.parse import urlparse

from .misc import MhtmlObject


class MhtmlParser:
	def __init__(self, filepath=None, raw_data=None):
		self.filepath = None

		if filepath:
			self.filepath = Path(filepath).resolve()
			raw_data = self.filepath.open('r').read()

		elif raw_data:
			pass

		else:
			raise ValueError('Did not specify a file path or MHTML string')

		self.html = None
		self.files = {}
		self._compiled_html = None

		self.parse_data(raw_data)


	def __str__(self):
		if self.filepath:
			return f'{self.__class__.__name__}(page="{self.html.location}", file="{self.filepath}")'

		return f'{self.__class__.__name__}(page="{self.html.location}")'


	@property
	def compiled_html(self):
		if self._compiled_html:
			return self._compiled_html

		obj_urls = []

		page = BeautifulSoup(self.html.body, 'html.parser')
		href_objs = page.find_all(True)

		for obj in href_objs:
			try:
				url = obj['href']

			except KeyError:
				try:
					url = obj['src']

				except KeyError:
					continue

			path = Path(urlparse(url).path)

			if not path.suffix:
				continue

			obj_urls.append(url)

			try:
				fileobj = self.files[str(path)]

			except KeyError as e:
				continue

			if path.suffix == '.css':
				new_obj = page.new_tag('style')
				new_obj['rel'] = 'stylesheet'

			elif path.suffix == '.js':
				new_obj = page.new_tag('script')

			elif any(map(fileobj.mimetype.startswith, ['audio', 'video', 'image'])):
				obj['src'] = f'data:{fileobj.mimetype};base64,{b64encode(fileobj.body).decode()}'
				continue

			else:
				continue

			new_obj['type'] = fileobj.mimetype
			new_obj.string = fileobj.body

			obj.replace_with(new_obj)

		self._compiled_html = page.prettify()
		return self._compiled_html


	def export(self, filepath=None):
		if not filepath:
			if not self.filepath:
				raise ValueError('Failed to specify an output filename') from None

			filepath = self.filepath.parent.joinpath(self.filepath.stem + '.html')

		with Path(filepath).open('w') as fd:
			fd.write(self.compiled_html)

		# logging module doesn't work for some reason
		print(f'Saved output to "{filepath}"')
		#logging.info(f'Saved output to "{filepath}"')


	def parse_data(self, raw_data):
		mdata = email.message_from_string(raw_data)

		if not mdata.is_multipart():
			raise TypeError('Not an MHTML file or missing MIME data') from None

		for idx, part in enumerate(mdata.walk()):
			if part.get_content_maintype() == "multipart":
				continue

			data = MhtmlObject(idx, part)

			if not any(map(data.location.startswith, ['http:', 'https:', 'data:'])):
				continue

			if data.mimetype == 'text/html' and not self.html:
				self.html = data

			else:
				self.files[str(data.path)] = data


class MhtmlFileParser(MhtmlParser):
	def __init__(self, filepath):
		super().__init__(filepath)


class MhtmlStringParser(MhtmlParser):
	def __init__(self, data):
		super().__init__(raw_data=data)

